@@ -18,6 +18,12 @@ You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
+@bug SMS Dialog issues on Hildon
+@bug Session timeouts are bad, possible solutions:
+	@li For every X minutes, if logged in, attempt login
+	@li Restructure so code handling login/dial/sms is beneath everything else and login attempts are made if those fail
+@todo Force login on connect if not already done
+@todo Can't text from dialpad (so can't do any arbitrary number texts)
 @todo Add logging support to make debugging issues for people a lot easier
 """
 
@@ -28,6 +34,9 @@ import sys
 import gc
 import os
 import threading
+import base64
+import ConfigParser
+import itertools
 import warnings
 import traceback
 
@@ -49,6 +58,15 @@ def getmtime_nothrow(path):
 		return 0
 
 
+def display_error_message(msg):
+	error_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, msg)
+
+	def close(dialog, response):
+		dialog.destroy()
+	error_dialog.connect("response", close)
+	error_dialog.run()
+
+
 class Dialcentral(object):
 
 	__pretty_app_name__ = "DialCentral"
@@ -74,13 +92,16 @@ class Dialcentral(object):
 	BACKENDS = (NULL_BACKEND, GC_BACKEND, GV_BACKEND)
 
 	_data_path = os.path.join(os.path.expanduser("~"), ".dialcentral")
+	_user_settings = "%s/settings.ini" % _data_path
 
 	def __init__(self):
+		self._initDone = False
 		self._connection = None
 		self._osso = None
 		self._clipboard = gtk.clipboard_get()
 
 		self._deviceIsOnline = True
+		self._credentials = ("", "")
 		self._selectedBackendId = self.NULL_BACKEND
 		self._defaultBackendId = self.GC_BACKEND
 		self._phoneBackends = None
@@ -95,26 +116,28 @@ class Dialcentral(object):
 				self._widgetTree = gtk.glade.XML(path)
 				break
 		else:
-			self.display_error_message("Cannot find dialcentral.glade")
+			display_error_message("Cannot find dialcentral.glade")
 			gtk.main_quit()
 			return
 
 		self._window = self._widgetTree.get_widget("mainWindow")
 		self._notebook = self._widgetTree.get_widget("notebook")
 		self._errorDisplay = gtk_toolbox.ErrorDisplay(self._widgetTree)
-		self._credentials = gtk_toolbox.LoginWindow(self._widgetTree)
+		self._credentialsDialog = gtk_toolbox.LoginWindow(self._widgetTree)
 
 		self._app = None
 		self._isFullScreen = False
 		if hildon is not None:
 			self._app = hildon.Program()
+			oldWindow = self._window
 			self._window = hildon.Window()
-			self._widgetTree.get_widget("vbox1").reparent(self._window)
+			oldWindow.get_child().reparent(self._window)
 			self._app.add_window(self._window)
 			self._widgetTree.get_widget("usernameentry").set_property('hildon-input-mode', 7)
 			self._widgetTree.get_widget("passwordentry").set_property('hildon-input-mode', 7|(1 << 29))
 			self._widgetTree.get_widget("callbackcombo").get_child().set_property('hildon-input-mode', (1 << 4))
 			hildon.hildon_helper_set_thumb_scrollbar(self._widgetTree.get_widget('recent_scrolledwindow'), True)
+			hildon.hildon_helper_set_thumb_scrollbar(self._widgetTree.get_widget('message_scrolledwindow'), True)
 			hildon.hildon_helper_set_thumb_scrollbar(self._widgetTree.get_widget('contacts_scrolledwindow'), True)
 
 			gtkMenu = self._widgetTree.get_widget("dialpad_menubar")
@@ -140,7 +163,7 @@ class Dialcentral(object):
 		self._widgetTree.signal_autoconnect(callbackMapping)
 
 		if self._window:
-			self._window.connect("destroy", gtk.main_quit)
+			self._window.connect("destroy", self._on_close)
 			self._window.show_all()
 			self._window.set_default_size(800, 300)
 
@@ -183,6 +206,7 @@ class Dialcentral(object):
 		else:
 			pass # warnings.warn("No OSSO", UserWarning)
 
+		# Setup maemo specifics
 		try:
 			import conic
 		except ImportError:
@@ -262,6 +286,7 @@ class Dialcentral(object):
 		for backendId in (self.GV_BACKEND, self.GC_BACKEND):
 			self._dialpads[backendId].dial = self._on_dial_clicked
 			self._recentViews[backendId].number_selected = self._on_number_selected
+			self._messagesViews[backendId].number_selected = self._on_number_selected
 			self._contactsViews[backendId].number_selected = self._on_number_selected
 
 			addressBooks = [
@@ -276,12 +301,20 @@ class Dialcentral(object):
 
 		callbackMapping = {
 			"on_paste": self._on_paste,
+			"on_refresh": self._on_refresh,
 			"on_clearcookies_clicked": self._on_clearcookies_clicked,
 			"on_notebook_switch_page": self._on_notebook_switch_page,
 			"on_about_activate": self._on_about_activate,
 		}
 		self._widgetTree.signal_autoconnect(callbackMapping)
 
+		self._initDone = True
+
+		config = ConfigParser.SafeConfigParser()
+		config.read(self._user_settings)
+		with gtk_toolbox.gtk_lock():
+			self.load_settings(config)
+
 		self.attempt_login(2)
 
 		return False
@@ -305,10 +338,28 @@ class Dialcentral(object):
 
 		loggedIn = False
 		try:
-			if self._phoneBackends[self._defaultBackendId].is_authed():
-				serviceId = self._defaultBackendId
-				loggedIn = True
-			for x in xrange(numOfAttempts):
+			username, password = self._credentials
+			serviceId = self._defaultBackendId
+
+			# Attempt using the cookies
+			loggedIn = self._phoneBackends[self._defaultBackendId].is_authed()
+			if loggedIn:
+				warnings.warn(
+					"Logged into %r through cookies" % self._phoneBackends[self._defaultBackendId],
+					UserWarning, 2
+				)
+
+			# Attempt using the settings file
+			if not loggedIn and username and password:
+				loggedIn = self._phoneBackends[self._defaultBackendId].login(username, password)
+				if loggedIn:
+					warnings.warn(
+						"Logged into %r through settings" % self._phoneBackends[self._defaultBackendId],
+						UserWarning, 2
+					)
+
+			# Query the user for credentials
+			for attemptCount in xrange(numOfAttempts):
 				if loggedIn:
 					break
 				with gtk_toolbox.gtk_lock():
@@ -316,33 +367,39 @@ class Dialcentral(object):
 						self.GV_BACKEND: "Google Voice",
 						self.GC_BACKEND: "Grand Central",
 					}
-					credentials = self._credentials.request_credentials_from(availableServices)
+					credentials = self._credentialsDialog.request_credentials_from(
+						availableServices, defaultCredentials = self._credentials
+					)
 					serviceId, username, password = credentials
 
 				loggedIn = self._phoneBackends[serviceId].login(username, password)
+			if 0 < attemptCount:
+				warnings.warn(
+					"Logged into %r through user request" % self._phoneBackends[serviceId],
+					UserWarning, 2
+				)
 		except RuntimeError, e:
 			warnings.warn(traceback.format_exc())
 			self._errorDisplay.push_exception_with_lock(e)
 
 		with gtk_toolbox.gtk_lock():
-			if not loggedIn:
+			if loggedIn:
+				self._credentials = username, password
+				self._change_loggedin_status(serviceId)
+			else:
 				self._errorDisplay.push_message("Login Failed")
-			self._change_loggedin_status(serviceId if loggedIn else self.NULL_BACKEND)
+				self._change_loggedin_status(self.NULL_BACKEND)
 		return loggedIn
 
-	def display_error_message(self, msg):
-		error_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, msg)
-
-		def close(dialog, response, editor):
-			editor.about_dialog = None
-			dialog.destroy()
-		error_dialog.connect("response", close, self)
-		error_dialog.run()
-
 	def _on_close(self, *args, **kwds):
-		if self._osso is not None:
-			self._osso.close()
-		gtk.main_quit()
+		try:
+			if self._osso is not None:
+				self._osso.close()
+
+			if self._initDone:
+				self._save_settings()
+		finally:
+			gtk.main_quit()
 
 	def _change_loggedin_status(self, newStatus):
 		oldStatus = self._selectedBackendId
@@ -367,6 +424,69 @@ class Dialcentral(object):
 
 		self._selectedBackendId = newStatus
 
+	def load_settings(self, config):
+		"""
+		@note UI Thread
+		"""
+		self._defaultBackendId = int(config.get(self.__pretty_app_name__, "active"))
+		try:
+			blobs = (
+				config.get(self.__pretty_app_name__, "bin_blob_%i" % i)
+				for i in xrange(len(self._credentials))
+			)
+			creds = (
+				base64.b64decode(blob)
+				for blob in blobs
+			)
+			self._credentials = tuple(creds)
+		except ConfigParser.NoSectionError, e:
+			warnings.warn(
+				"Settings file %s is missing section %s" % (
+					self._user_settings,
+					e.section,
+				),
+				stacklevel=2
+			)
+
+		for backendId, view in itertools.chain(
+			self._dialpads.iteritems(),
+			self._accountViews.iteritems(),
+			self._messagesViews.iteritems(),
+			self._recentViews.iteritems(),
+			self._contactsViews.iteritems(),
+		):
+			sectionName = "%s - %s" % (backendId, view.name())
+			try:
+				view.load_settings(config, sectionName)
+			except ConfigParser.NoSectionError, e:
+				warnings.warn(
+					"Settings file %s is missing section %s" % (
+						self._user_settings,
+						e.section,
+					),
+					stacklevel=2
+				)
+
+	def save_settings(self, config):
+		"""
+		@note Thread Agnostic
+		"""
+		config.add_section(self.__pretty_app_name__)
+		config.set(self.__pretty_app_name__, "active", str(self._selectedBackendId))
+		for i, value in enumerate(self._credentials):
+			blob = base64.b64encode(value)
+			config.set(self.__pretty_app_name__, "bin_blob_%i" % i, blob)
+		for backendId, view in itertools.chain(
+			self._dialpads.iteritems(),
+			self._accountViews.iteritems(),
+			self._messagesViews.iteritems(),
+			self._recentViews.iteritems(),
+			self._contactsViews.iteritems(),
+		):
+			sectionName = "%s - %s" % (backendId, view.name())
+			config.add_section(sectionName)
+			view.save_settings(config, sectionName)
+
 	def _guess_preferred_backend(self, backendAndCookiePaths):
 		modTimeAndPath = [
 			(getmtime_nothrow(path), backendId, path)
@@ -375,6 +495,15 @@ class Dialcentral(object):
 		modTimeAndPath.sort()
 		return modTimeAndPath[-1][1]
 
+	def _save_settings(self):
+		"""
+		@note Thread Agnostic
+		"""
+		config = ConfigParser.SafeConfigParser()
+		self.save_settings(config)
+		with open(self._user_settings, "wb") as configFile:
+			config.write(configFile)
+
 	def _on_device_state_change(self, shutdown, save_unsaved_data, memory_low, system_inactivity, message, userData):
 		"""
 		For shutdown or save_unsaved_data, our only state is cookies and I think the cookie manager handles that for us.
@@ -388,6 +517,9 @@ class Dialcentral(object):
 			self._contactsViews[self._selectedBackendId].clear_caches()
 			gc.collect()
 
+		if save_unsaved_data or shutdown:
+			self._save_settings()
+
 	def _on_connection_change(self, connection, event, magicIdentifier):
 		"""
 		@note Hildon specific
@@ -401,13 +533,15 @@ class Dialcentral(object):
 
 		if status == conic.STATUS_CONNECTED:
 			self._deviceIsOnline = True
-			backgroundLogin = threading.Thread(target=self.attempt_login, args=[2])
-			backgroundLogin.setDaemon(True)
-			backgroundLogin.start()
+			if self._initDone:
+				backgroundLogin = threading.Thread(target=self.attempt_login, args=[2])
+				backgroundLogin.setDaemon(True)
+				backgroundLogin.start()
 		elif status == conic.STATUS_DISCONNECTED:
 			self._deviceIsOnline = False
-			self._defaultBackendId = self._selectedBackendId
-			self._change_loggedin_status(self.NULL_BACKEND)
+			if self._initDone:
+				self._defaultBackendId = self._selectedBackendId
+				self._change_loggedin_status(self.NULL_BACKEND)
 
 	def _on_window_state_change(self, widget, event, *args):
 		"""
@@ -441,12 +575,14 @@ class Dialcentral(object):
 		backgroundLogin.start()
 
 	def _on_notebook_switch_page(self, notebook, page, page_num):
-		if page_num == self.CONTACTS_TAB:
-			self._contactsViews[self._selectedBackendId].update()
-		elif page_num == self.RECENT_TAB:
+		if page_num == self.RECENT_TAB:
 			self._recentViews[self._selectedBackendId].update()
 		elif page_num == self.MESSAGES_TAB:
 			self._messagesViews[self._selectedBackendId].update()
+		elif page_num == self.CONTACTS_TAB:
+			self._contactsViews[self._selectedBackendId].update()
+		elif page_num == self.ACCOUNT_TAB:
+			self._accountViews[self._selectedBackendId].update()
 
 		tabTitle = self._notebook.get_tab_label(self._notebook.get_nth_page(page_num)).get_text()
 		if hildon is not None:
@@ -457,7 +593,7 @@ class Dialcentral(object):
 	def _on_number_selected(self, action, number, message):
 		if action == "select":
 			self._dialpads[self._selectedBackendId].set_number(number)
-			self._notebook.set_current_page(0)
+			self._notebook.set_current_page(self.KEYPAD_TAB)
 		elif action == "dial":
 			self._on_dial_clicked(number)
 		elif action == "sms":
@@ -523,7 +659,15 @@ class Dialcentral(object):
 
 		if dialed:
 			self._dialpads[self._selectedBackendId].clear()
-			self._recentViews[self._selectedBackendId].clear()
+
+	def _on_refresh(self, *args):
+		page_num = self._notebook.get_current_page()
+		if page_num == self.CONTACTS_TAB:
+			self._contactsViews[self._selectedBackendId].update(force=True)
+		elif page_num == self.RECENT_TAB:
+			self._recentViews[self._selectedBackendId].update(force=True)
+		elif page_num == self.MESSAGES_TAB:
+			self._messagesViews[self._selectedBackendId].update(force=True)
 
 	def _on_paste(self, *args):
 		contents = self._clipboard.wait_for_text()
