@@ -22,7 +22,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 from __future__ import with_statement
 
 import threading
-import time
 import warnings
 
 import gobject
@@ -341,7 +340,7 @@ class PhoneTypeSelector(object):
 
 		self._action = self.ACTION_CANCEL
 
-	def run(self, contactDetails, message = ""):
+	def run(self, contactDetails, message = "", parent = None):
 		self._action = self.ACTION_CANCEL
 		self._typemodel.clear()
 		self._typeview.set_model(self._typemodel)
@@ -372,7 +371,13 @@ class PhoneTypeSelector(object):
 			self._message.set_markup("")
 			self._message.hide()
 
-		userResponse = self._dialog.run()
+		if parent is not None:
+			self._dialog.set_transient_for(parent)
+
+		try:
+			userResponse = self._dialog.run()
+		finally:
+			self._dialog.hide()
 
 		if userResponse == gtk.RESPONSE_OK:
 			phoneNumber = self._get_number()
@@ -383,7 +388,7 @@ class PhoneTypeSelector(object):
 			self._action = self.ACTION_CANCEL
 
 		if self._action == self.ACTION_SEND_SMS:
-			smsMessage = self._smsDialog.run(phoneNumber, message)
+			smsMessage = self._smsDialog.run(phoneNumber, message, parent)
 			if not smsMessage:
 				phoneNumber = ""
 				self._action = self.ACTION_CANCEL
@@ -394,7 +399,7 @@ class PhoneTypeSelector(object):
 		self._typeview.remove_column(numberColumn)
 		self._typeview.remove_column(typeColumn)
 		self._typeview.set_model(None)
-		self._dialog.hide()
+
 		return self._action, phoneNumber, smsMessage
 
 	def _get_number(self):
@@ -442,7 +447,7 @@ class SmsEntryDialog(object):
 		self._smsEntry = self._widgetTree.get_widget("smsEntry")
 		self._smsEntry.get_buffer().connect("changed", self._on_entry_changed)
 
-	def run(self, number, message = ""):
+	def run(self, number, message = "", parent = None):
 		if message:
 			self._message.set_markup(message)
 			self._message.show()
@@ -452,7 +457,14 @@ class SmsEntryDialog(object):
 		self._smsEntry.get_buffer().set_text("")
 		self._update_letter_count()
 
-		userResponse = self._dialog.run()
+		if parent is not None:
+			self._dialog.set_transient_for(parent)
+
+		try:
+			userResponse = self._dialog.run()
+		finally:
+			self._dialog.hide()
+
 		if userResponse == gtk.RESPONSE_OK:
 			entryBuffer = self._smsEntry.get_buffer()
 			enteredMessage = entryBuffer.get_text(entryBuffer.get_start_iter(), entryBuffer.get_end_iter())
@@ -460,7 +472,6 @@ class SmsEntryDialog(object):
 		else:
 			enteredMessage = ""
 
-		self._dialog.hide()
 		return enteredMessage
 
 	def _update_letter_count(self, *args):
@@ -531,6 +542,19 @@ class Dialpad(object):
 	def clear(self):
 		self.set_number("")
 
+	@staticmethod
+	def name():
+		return "Dialpad"
+
+	def load_settings(self, config, section):
+		pass
+
+	def save_settings(self, config, section):
+		"""
+		@note Thread Agnostic
+		"""
+		pass
+
 	def _on_dial_clicked(self, widget):
 		self.dial(self.get_number())
 
@@ -561,23 +585,28 @@ class AccountInfo(object):
 	def __init__(self, widgetTree, backend, errorDisplay):
 		self._errorDisplay = errorDisplay
 		self._backend = backend
+		self._isPopulated = False
 
 		self._callbackList = gtk.ListStore(gobject.TYPE_STRING)
 		self._accountViewNumberDisplay = widgetTree.get_widget("gcnumber_display")
 		self._callbackCombo = widgetTree.get_widget("callbackcombo")
 		self._onCallbackentryChangedId = 0
 
+		self._defaultCallback = ""
+
 	def enable(self):
 		assert self._backend.is_authed()
 		self._accountViewNumberDisplay.set_use_markup(True)
 		self.set_account_number("")
 		self._callbackList.clear()
-		self.update()
 		self._onCallbackentryChangedId = self._callbackCombo.get_child().connect("changed", self._on_callbackentry_changed)
+		self.update(force=True)
 
 	def disable(self):
 		self._callbackCombo.get_child().disconnect(self._onCallbackentryChangedId)
+
 		self.clear()
+
 		self._callbackList.clear()
 
 	def get_selected_callback_number(self):
@@ -589,20 +618,39 @@ class AccountInfo(object):
 		"""
 		self._accountViewNumberDisplay.set_label("<span size='23000' weight='bold'>%s</span>" % (number))
 
-	def update(self):
-		self.populate_callback_combo()
+	def update(self, force = False):
+		if not force and self._isPopulated:
+			return
+		self._populate_callback_combo()
 		self.set_account_number(self._backend.get_account_number())
 
 	def clear(self):
 		self._callbackCombo.get_child().set_text("")
 		self.set_account_number("")
+		self._isPopulated = False
 
-	def populate_callback_combo(self):
+	@staticmethod
+	def name():
+		return "Account Info"
+
+	def load_settings(self, config, section):
+		self._defaultCallback = config.get(section, "callback")
+
+	def save_settings(self, config, section):
+		"""
+		@note Thread Agnostic
+		"""
+		callback = self.get_selected_callback_number()
+		config.set(section, "callback", callback)
+
+	def _populate_callback_combo(self):
+		self._isPopulated = True
 		self._callbackList.clear()
 		try:
 			callbackNumbers = self._backend.get_callback_numbers()
 		except RuntimeError, e:
 			self._errorDisplay.push_exception(e)
+			self._isPopulated = False
 			return
 
 		for number, description in callbackNumbers.iteritems():
@@ -610,27 +658,31 @@ class AccountInfo(object):
 
 		self._callbackCombo.set_model(self._callbackList)
 		self._callbackCombo.set_text_column(0)
+		#callbackNumber = self._backend.get_callback_number()
+		callbackNumber = self._defaultCallback
+		self._callbackCombo.get_child().set_text(make_pretty(callbackNumber))
+
+	def _set_callback_number(self, number):
+		"""
+		@todo Potential blocking on web access, maybe we should defer this or put up a dialog?
+		"""
 		try:
-			callbackNumber = self._backend.get_callback_number()
+			if not self._backend.is_valid_syntax(number):
+				self._errorDisplay.push_message("%s is not a valid callback number" % number)
+			elif number == self._backend.get_callback_number():
+				warnings.warn("Callback number already is %s" % self._backend.get_callback_number(), UserWarning, 2)
+			else:
+				self._backend.set_callback_number(number)
+				warnings.warn("Callback number set to %s" % self._backend.get_callback_number(), UserWarning, 2)
 		except RuntimeError, e:
 			self._errorDisplay.push_exception(e)
-			return
-		self._callbackCombo.get_child().set_text(make_pretty(callbackNumber))
 
 	def _on_callbackentry_changed(self, *args):
 		"""
 		@todo Potential blocking on web access, maybe we should defer this or put up a dialog?
 		"""
-		try:
-			text = self.get_selected_callback_number()
-			if not self._backend.is_valid_syntax(text):
-				self._errorDisplay.push_message("%s is not a valid callback number" % text)
-			elif text == self._backend.get_callback_number():
-				warnings.warn("Callback number already is %s" % self._backend.get_callback_number(), UserWarning, 2)
-			else:
-				self._backend.set_callback_number(text)
-		except RuntimeError, e:
-			self._errorDisplay.push_exception(e)
+		text = self.get_selected_callback_number()
+		self._set_callback_number(text)
 
 
 class RecentCallsView(object):
@@ -639,7 +691,7 @@ class RecentCallsView(object):
 		self._errorDisplay = errorDisplay
 		self._backend = backend
 
-		self._recenttime = 0.0
+		self._isPopulated = False
 		self._recentmodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
 		self._recentview = widgetTree.get_widget("recentview")
 		self._recentviewselection = None
@@ -652,6 +704,7 @@ class RecentCallsView(object):
 		self._recentviewColumn.add_attribute(textrenderer, "text", 1)
 		self._recentviewColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
+		self._window = gtk_toolbox.find_parent_window(self._recentview)
 		self._phoneTypeSelector = PhoneTypeSelector(widgetTree, self._backend)
 
 	def enable(self):
@@ -666,6 +719,9 @@ class RecentCallsView(object):
 
 	def disable(self):
 		self._recentview.disconnect(self._onRecentviewRowActivatedId)
+
+		self.clear()
+
 		self._recentview.remove_column(self._recentviewColumn)
 		self._recentview.set_model(None)
 
@@ -675,26 +731,39 @@ class RecentCallsView(object):
 		"""
 		raise NotImplementedError
 
-	def update(self):
-		if (time.time() - self._recenttime) < 300:
+	def update(self, force = False):
+		if not force and self._isPopulated:
 			return
 		backgroundPopulate = threading.Thread(target=self._idly_populate_recentview)
 		backgroundPopulate.setDaemon(True)
 		backgroundPopulate.start()
 
 	def clear(self):
-		self._recenttime = 0.0
+		self._isPopulated = False
 		self._recentmodel.clear()
 
+	@staticmethod
+	def name():
+		return "Recent Calls"
+
+	def load_settings(self, config, section):
+		pass
+
+	def save_settings(self, config, section):
+		"""
+		@note Thread Agnostic
+		"""
+		pass
+
 	def _idly_populate_recentview(self):
-		self._recenttime = time.time()
+		self._isPopulated = True
 		self._recentmodel.clear()
 
 		try:
 			recentItems = self._backend.get_recent()
 		except RuntimeError, e:
 			self._errorDisplay.push_exception_with_lock(e)
-			self._recenttime = 0.0
+			self._isPopulated = False
 			recentItems = []
 
 		for personsName, phoneNumber, date, action in recentItems:
@@ -715,7 +784,11 @@ class RecentCallsView(object):
 		contactPhoneNumbers = [("Phone", number)]
 		description = self._recentmodel.get_value(itr, 1)
 
-		action, phoneNumber, message = self._phoneTypeSelector.run(contactPhoneNumbers, message = description)
+		action, phoneNumber, message = self._phoneTypeSelector.run(
+			contactPhoneNumbers,
+			message = description,
+			parent = self._window,
+		)
 		if action == PhoneTypeSelector.ACTION_CANCEL:
 			return
 		assert phoneNumber
@@ -730,7 +803,7 @@ class MessagesView(object):
 		self._errorDisplay = errorDisplay
 		self._backend = backend
 
-		self._messagetime = 0.0
+		self._isPopulated = False
 		self._messagemodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
 		self._messageview = widgetTree.get_widget("messages_view")
 		self._messageviewselection = None
@@ -743,6 +816,7 @@ class MessagesView(object):
 		self._messageviewColumn.add_attribute(textrenderer, "markup", 1)
 		self._messageviewColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
+		self._window = gtk_toolbox.find_parent_window(self._messageview)
 		self._phoneTypeSelector = PhoneTypeSelector(widgetTree, self._backend)
 
 	def enable(self):
@@ -757,6 +831,9 @@ class MessagesView(object):
 
 	def disable(self):
 		self._messageview.disconnect(self._onMessageviewRowActivatedId)
+
+		self.clear()
+
 		self._messageview.remove_column(self._messageviewColumn)
 		self._messageview.set_model(None)
 
@@ -766,26 +843,39 @@ class MessagesView(object):
 		"""
 		raise NotImplementedError
 
-	def update(self):
-		if (time.time() - self._messagetime) < 300:
+	def update(self, force = False):
+		if not force and self._isPopulated:
 			return
 		backgroundPopulate = threading.Thread(target=self._idly_populate_messageview)
 		backgroundPopulate.setDaemon(True)
 		backgroundPopulate.start()
 
 	def clear(self):
-		self._messagetime = 0.0
+		self._isPopulated = False
 		self._messagemodel.clear()
 
+	@staticmethod
+	def name():
+		return "Messages"
+
+	def load_settings(self, config, section):
+		pass
+
+	def save_settings(self, config, section):
+		"""
+		@note Thread Agnostic
+		"""
+		pass
+
 	def _idly_populate_messageview(self):
-		self._messagetime = time.time()
+		self._isPopulated = True
 		self._messagemodel.clear()
 
 		try:
 			messageItems = self._backend.get_messages()
 		except RuntimeError, e:
 			self._errorDisplay.push_exception_with_lock(e)
-			self._messagetime = 0.0
+			self._isPopulated = False
 			messageItems = []
 
 		for header, number, relativeDate, message in messageItems:
@@ -804,7 +894,11 @@ class MessagesView(object):
 		contactPhoneNumbers = [("Phone", self._messagemodel.get_value(itr, 0))]
 		description = self._messagemodel.get_value(itr, 1)
 
-		action, phoneNumber, message = self._phoneTypeSelector.run(contactPhoneNumbers, message = description)
+		action, phoneNumber, message = self._phoneTypeSelector.run(
+			contactPhoneNumbers,
+			message = description,
+			parent = self._window,
+		)
 		if action == PhoneTypeSelector.ACTION_CANCEL:
 			return
 		assert phoneNumber
@@ -825,7 +919,7 @@ class ContactsView(object):
 		self._booksList = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
 		self._booksSelectionBox = widgetTree.get_widget("addressbook_combo")
 
-		self._contactstime = 0.0
+		self._isPopulated = False
 		self._contactsmodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
 		self._contactsviewselection = None
 		self._contactsview = widgetTree.get_widget("contactsview")
@@ -848,6 +942,7 @@ class ContactsView(object):
 
 		self._onContactsviewRowActivatedId = 0
 		self._onAddressbookComboChangedId = 0
+		self._window = gtk_toolbox.find_parent_window(self._contactsview)
 		self._phoneTypeSelector = PhoneTypeSelector(widgetTree, self._backend)
 
 	def enable(self):
@@ -884,6 +979,8 @@ class ContactsView(object):
 		self._contactsview.disconnect(self._onContactsviewRowActivatedId)
 		self._booksSelectionBox.disconnect(self._onAddressbookComboChangedId)
 
+		self.clear()
+
 		self._booksSelectionBox.clear()
 		self._booksSelectionBox.set_model(None)
 		self._contactsview.set_model(None)
@@ -905,20 +1002,17 @@ class ContactsView(object):
 
 	def open_addressbook(self, bookFactoryId, bookId):
 		self._addressBook = self._addressBookFactories[bookFactoryId].open_addressbook(bookId)
-		self._contactstime = 0
-		backgroundPopulate = threading.Thread(target=self._idly_populate_contactsview)
-		backgroundPopulate.setDaemon(True)
-		backgroundPopulate.start()
+		self.update(force=True)
 
-	def update(self):
-		if (time.time() - self._contactstime) < 300:
+	def update(self, force = False):
+		if not force and self._isPopulated:
 			return
 		backgroundPopulate = threading.Thread(target=self._idly_populate_contactsview)
 		backgroundPopulate.setDaemon(True)
 		backgroundPopulate.start()
 
 	def clear(self):
-		self._contactstime = 0.0
+		self._isPopulated = False
 		self._contactsmodel.clear()
 
 	def clear_caches(self):
@@ -932,8 +1026,21 @@ class ContactsView(object):
 	def extend(self, books):
 		self._addressBookFactories.extend(books)
 
+	@staticmethod
+	def name():
+		return "Contacts"
+
+	def load_settings(self, config, section):
+		pass
+
+	def save_settings(self, config, section):
+		"""
+		@note Thread Agnostic
+		"""
+		pass
+
 	def _idly_populate_contactsview(self):
-		#@todo Add a lock so only one code path can be in here at a time
+		self._isPopulated = True
 		self.clear()
 
 		# completely disable updating the treeview while we populate the data
@@ -945,7 +1052,7 @@ class ContactsView(object):
 			contacts = addressBook.get_contacts()
 		except RuntimeError, e:
 			contacts = []
-			self._contactstime = 0.0
+			self._isPopulated = False
 			self._errorDisplay.push_exception_with_lock(e)
 		for contactId, contactName in contacts:
 			contactType = (addressBook.contact_source_short_name(contactId), )
@@ -975,14 +1082,17 @@ class ContactsView(object):
 			contactDetails = self._addressBook.get_contact_details(contactId)
 		except RuntimeError, e:
 			contactDetails = []
-			self._contactstime = 0.0
 			self._errorDisplay.push_exception(e)
 		contactPhoneNumbers = [phoneNumber for phoneNumber in contactDetails]
 
 		if len(contactPhoneNumbers) == 0:
 			return
 
-		action, phoneNumber, message = self._phoneTypeSelector.run(contactPhoneNumbers, message = contactName)
+		action, phoneNumber, message = self._phoneTypeSelector.run(
+			contactPhoneNumbers,
+			message = contactName,
+			parent = self._window,
+		)
 		if action == PhoneTypeSelector.ACTION_CANCEL:
 			return
 		assert phoneNumber
